==> em_tasks-0.1.64-py2.py3-none-any.whl: em_tasks/utils/product_converters.py <==
# -*- coding: utf-8 -*-

import os

from pint import UnitRegistry

from em_tasks import logger


class AmzProductConverter():
  def __init__(self, marketplace='us'):
    self.ureg = UnitRegistry()
    self.source = 'AMZ_{}'.format(marketplace.upper())
    self.code = ''
    self.host = ''
    marketplace = marketplace.lower()
    if marketplace == 'jp':
      self.code = 'X04'
      self.host = 'https://www.amazon.co.jp'
    elif marketplace == 'uk':
      self.code = 'X05'
      self.host = 'https://www.amazon.co.uk'
    elif marketplace == 'us':
      self.code = 'X06'
      self.host = 'https://www.amazon.com'
    elif marketplace == 'tr':
      self.code = 'X07'
      self.host = 'https://www.amazon.com.tr'
    elif marketplace == 'de':
      self.code = 'X08'
      self.host = 'https://www.amazon.de'

  def convert(self, data):
    return {
      'source': self.source,
      'source_product_id': data['asin'],
      'source_product_url': '{}/dp/{}'.format(self.host, data['asin']),
      'has_only_default_variant': True,
      'existances': True,
      'title': data['title'],
      'description': self.get_description(data),
      'brand': self.get_brand(data),
      'categories': self.get_categories(data),
      'images': ';'.join(self.get_images(data)),
      'specifications': self.get_specifications(data),
      'options': [],
      'sku': '{}-{}'.format(self.code, data['asin']),
      'upc': self.get_upc(data),
      'weight': self.get_weight(data),
      'returnable': None,
    }

  def get_description(self, product_info):
    d = dict()
    attrs = product_info.get('attributes', d)
    desc = attrs.get('product_description', [d])[0].get('value', product_info.get('title', ''))

    features = product_info.get('attributes', d).get('bullet_point', [])
    features_list = []
    for feature in features:
      bullet = feature.get('value', None)
      if not bullet:
        continue

      features_list.append('<li>{}</li>'.format(bullet))

    feature_s = '<ul>{}</ul>'.format("\n".join(features_list))

    specs = []
    specifications = self.get_specifications(product_info)
    if specifications:
      for spec in specifications:
        specs.append('<li>{}: {}</li>'.format(spec['name'], spec['value']))
    specs_s = '<ul>{}</ul>'.format("\n".join(specs))

    return '{}<br />{}<br />{}'.format(feature_s, specs_s, desc)

  def get_brand(self, product_info):
    if 'brand' in product_info and product_info['brand']:
      brand = product_info['brand']
    else:
      attrs = product_info.get('attributes', dict())
      brand = product_info.get(
        'brand', attrs.get('Brand', dict()).get('value', None))
    if brand:
      brand = brand.replace('Visit the', '').replace('Amazon', '').replace('amazon', '').strip()

    return brand

  def get_upc(self, product_info):
    upc = None
    for identifiers in product_info.get('identifiers', []):
      for identifier in identifiers.get('identifiers', []):
        if upc is None:
          upc = identifier.get('identifier', None)

        identifier_type = identifier.get('identifierType', '')
        if identifier_type == 'GTIN' or identifier_type == 'UPC':
          upc = identifier.get('identifier', upc)
          break

    return upc

  def get_categories(self, product_info):
    categories = None
    d = dict()
    summary = product_info.get('summaries', [d])[0]
    if summary and 'websiteDisplayGroupName' in summary and summary['websiteDisplayGroupName']:
      categories = summary['websiteDisplayGroupName']

    return categories

  def get_specifications(self, product_info):
    specifications = []
    specs = product_info.get('specifications', product_info.get('key_specs', []))
    for spec in specs:
      if not isinstance(spec, dict):
        continue

      spec_name = spec['name'].lower()
      if spec_name.find('asin') != -1 or spec_name.find('rank') != -1 or spec_name.find('first available') != -1 or spec_name.find('amazon') != -1:
        continue

      specifications.append({'name': spec['name'], 'value': spec['value']})

    summaries = {
      'color': 'Color',
      'manufacturer': 'Manufacturer',
      'modelNumber': 'Model Number',
      'packageQuantity': 'Package Quantity',
      'partNumber': 'Part Number',
      # 'releaseDate': 'Release Date',
      'size': 'Size',
      'style': 'Style'
    }
    for summary in product_info.get('summaries', []):
      for k, v in summary.items():
        if k in summaries:
          specifications.append({'name': summaries[k], 'value': v})

    attrs = product_info.get('attributes', {})
    attr_keys = {
      'item_form': 'Form',
      'target_gender': 'Gender',
      'ingredients': 'Ingredients',
      'directions': 'Directions'
    }
    for k, v in attr_keys.items():
      if k in attrs:
        try:
          attr = attrs[k]
          if 'value' in attr and attr['value']:
            specifications.append({'name': v, 'value': attr[0]['value']})
        except Exception as e:
          logger.exception(e)
          logger.info('[AttributeError] Key: %s, Attr: %s', k, attr)

    return specifications

  def get_images(self, product_info):
    images = product_info.get('images', [])

    if isinstance(images, str):
      images = [images]

    imgs = dict()
    for image in images:
      img_url = self.opt_img(image)
      if img_url in imgs:
        imgs[img_url] += 1
      else:
        imgs[img_url] = 1

    imgs_origin = [img_url for img_url, cnt in imgs.items() if cnt == 1]
    imgs_resized = [img_url for img_url, cnt in imgs.items() if cnt > 1]

    return imgs_origin if imgs_origin else imgs_resized

  def get_weight_from_dimensions(self, dimensions):
    if len(dimensions) > 0:
      di = dimensions[0]
      return di.get('package', di.get('item', {})).get('weight', {})

  def get_weight_from_attr(self, attrs):
    return attrs.get('item_package_weight', attrs.get('item_weight', []))

  def get_weight(self, product_info):
    d = dict()
    attrs = product_info.get('attributes', d)
    dimensions = product_info.get('dimensions', [])
    weight = self.get_weight_from_dimensions(dimensions) or self.get_weight_from_attr(attrs)
    if not weight:
      return

    weight_in_pounds = None
    weight = weight[0] if isinstance(weight, list) else weight
    weight_str = '{} {}'.format(weight.get('value'), weight.get('unit'))
    try:
      weight_in_pounds = round(self.ureg(weight_str).to('lbs').m, 2) or 0.1
    except Exception as e:
      weight_in_pounds = None

    return weight_in_pounds

  def get_dimensions(self, product_info):
    d = dict()
    ds = {}
    raw_dimensions = product_info.get('dimensions', [{}])[0]
    dimensions = product_info.get('item_package_dimensions', product_info.get('attributes', d).get('item_package_dimensions', raw_dimensions.get('package', raw_dimensions.get('item', {}))))
    if isinstance(dimensions, list):
      dimensions = dimensions[0]
    if not isinstance(dimensions, dict):
      return {'width': None, 'height': None, 'length': None}

    width = dimensions.get('width', d)
    if width:
      width_str = '{} {}'.format(width['value'], width['unit'])
      try:
        ds['width'] = round(self.ureg(width_str).to('inch').m, 2)
      except:
        ds['width'] = None
    else:
      ds['width'] = None

    height = dimensions.get('height', d)
    if height:
      height_str = '{} {}'.format(height['value'], height['unit'])
      try:
        ds['height'] = round(self.ureg(height_str).to('inch').m, 2)
      except:
        ds['height'] = None
    else:
      ds['height'] = None

    length = dimensions.get('length', d)
    if length:
      length_str = '{} {}'.format(length['value'], length['unit'])
      try:
        ds['length'] = round(self.ureg(length_str).to('inch').m, 2)
      except:
        ds['length'] = None
    else:
      ds['length'] = None

    return ds

  def get_sales_rank(self, product_info):
    return product_info.get('sales_rank', None)

  def is_fragile(self, product_info):
    attrs = product_info.get('attributes', {})
    if 'is_fragile' not in attrs or not attrs['is_fragile']:
      return False

    is_fragile = False
    for entry in attrs['is_fragile']:
      is_fragile = entry.get('value', False)
      if is_fragile:
        break

    return is_fragile

  def is_batteries_included(self, product_info):
    attrs = product_info.get('attributes', {})
    if 'batteries_included' not in attrs or not attrs['batteries_included']:
      return False

    batteries_required = False
    for entry in attrs.get('batteries_required', []):
      batteries_required = entry.get('value', False)
      if batteries_required:
        break

    if not batteries_required:
      return False

    batteries_included = False
    for entry in attrs['batteries_included']:
      batteries_included = entry.get('value', False)
      if batteries_included:
        break

    return batteries_included

  def is_contains_food_or_beverage(self, product_info):
    attrs = product_info.get('attributes', {})
    if 'contains_food_or_beverage' not in attrs or not attrs['contains_food_or_beverage']:
      return False

    contains_food_or_beverage = False
    for entry in attrs['contains_food_or_beverage']:
      contains_food_or_beverage = entry.get('value', False)
      if contains_food_or_beverage:
        break

    return contains_food_or_beverage

  def is_liquid(self, product_info):
    attrs = product_info.get('attributes', {})
    if 'item_form' not in attrs or not attrs['item_form']:
      return False

    contains_liquid = False
    for entry in attrs['item_form']:
      item_form = entry.get('value', '').lower()
      if item_form == 'liquid':
        contains_liquid = True
        break

    return contains_liquid

  def is_alcohol(self, product_info):
    attrs = product_info.get('attributes', {})
    if 'alcohol_content' not in attrs or not attrs['alcohol_content']:
      return False

    contains_alcohol = False
    for entry in attrs['alcohol_content']:
      try:
        alcohol_percentage = float(entry.get('value', 0))
        if alcohol_percentage >= 3:
          contains_alcohol = True
          break
      except:
        pass

    return contains_alcohol

  def is_plant_or_seed(self, product_info):
    product_types = product_info.get('productTypes', product_info.get('attributes', {}).get('productTypes', None))
    if not product_types:
      title = product_info.get('title', '').lower()
      if title.find('plant') != -1:
        return True

      return False

    plant_or_seed = False
    for product_type in product_types:
      try:
        product_type_s = product_type.get('productType', product_type.get('value', '')).lower()
        if product_type_s.find('plant') != -1 or product_type_s.find('seed') != -1:
          plant_or_seed = True
          break
      except:
        pass

    return plant_or_seed

  def is_test_strip(self, product_info):
    title = product_info.get('title', '').lower()
    if title.find('testing kit') != -1 or title.find('test kit') != -1:
      return True

    product_types = product_info.get('productTypes', product_info.get('attributes', {}).get('productTypes', None))
    if not product_types:
      return False

    test_strip = False
    for product_type in product_types:
      product_type_val = product_type.get('productType', product_type.get('value', ''))
      if product_type_val.upper().find('TEST_STRIP') != -1:
        test_strip = True
        break

    return test_strip

  def is_spray(self, product_info):
    title = product_info.get('title', '').lower()
    if title.find('spray') != -1:
      return True

    item_forms = product_info.get('attributes', {}).get('item_form', None)
    if not item_forms:
      return False

    spray = False
    for item_form in item_forms:
      item_form_val = item_form.get('value', '')
      if item_form_val and item_form_val.find('Spray') != -1:
        spray = True
        break

    return spray

  def opt_img(self, img):
    if '._' in img:
      filename = os.path.basename(img)
      parts = filename.split('.')
      filename = parts[0] + '.' + parts[len(parts) - 1]
      img = os.path.dirname(img) + '/' + filename

    return img

==> em_tasks-0.1.65-py2.py3-none-any.whl: em_tasks/utils/product_converters.py <==
# -*- coding: utf-8 -*-

import os

from pint import UnitRegistry

from em_tasks import logger


class AmzProductConverter():
  def __init__(self, marketplace='us'):
    self.ureg = UnitRegistry()
    self.source = 'AMZ_{}'.format(marketplace.upper())
    self.code = ''
    self.host = ''
    marketplace = marketplace.lower()
    if marketplace == 'jp':
      self.code = 'X04'
      self.host = 'https://www.amazon.co.jp'
    elif marketplace == 'uk':
      self.code = 'X05'
      self.host = 'https://www.amazon.co.uk'
    elif marketplace == 'us':
      self.code = 'X06'
      self.host = 'https://www.amazon.com'
    elif marketplace == 'tr':
      self.code = 'X07'
      self.host = 'https://www.amazon.com.tr'
    elif marketplace == 'de':
      self.code = 'X08'
      self.host = 'https://www.amazon.de'

  def convert(self, data):
    return {
      'source': self.source,
      'source_product_id': data['asin'],
      'source_product_url': '{}/dp/{}'.format(self.host, data['asin']),
      'has_only_default_variant': True,
      'existances': True,
      'title': data['title'],
      'description': self.get_description(data),
      'brand': self.get_brand(data),
      'categories': self.get_categories(data),
      'images': ';'.join(self.get_images(data)),
      'specifications': self.get_specifications(data),
      'options': [],
      'sku': '{}-{}'.format(self.code, data['asin']),
      'upc': self.get_upc(data),
      'weight': self.get_weight(data),
      'returnable': None,
    }

  def get_description(self, product_info):
    d = dict()
    attrs = product_info.get('attributes', d)
    desc = attrs.get('product_description', [d])[0].get('value', product_info.get('title', ''))

    features = product_info.get('attributes', d).get('bullet_point', [])
    features_list = []
    for feature in features:
      bullet = feature.get('value', None)
      if not bullet:
        continue

      features_list.append('<li>{}</li>'.format(bullet))

    feature_s = '<ul>{}</ul>'.format("\n".join(features_list))

    specs = []
    specifications = self.get_specifications(product_info)
    if specifications:
      for spec in specifications:
        specs.append('<li>{}: {}</li>'.format(spec['name'], spec['value']))
    specs_s = '<ul>{}</ul>'.format("\n".join(specs))

    return '{}<br />{}<br />{}'.format(feature_s, specs_s, desc)

  def get_brand(self, product_info):
    if 'brand' in product_info and product_info['brand']:
      brand = product_info['brand']
    else:
      attrs = product_info.get('attributes', dict())
      brand = product_info.get(
        'brand', attrs.get('Brand', dict()).get('value', None))
    if brand:
      brand = brand.replace('Visit the', '').replace('Amazon', '').replace('amazon', '').strip()

    return brand

  def get_upc(self, product_info):
    upc = None
    for identifiers in product_info.get('identifiers', []):
      for identifier in identifiers.get('identifiers', []):
        if upc is None:
          upc = identifier.get('identifier', None)

        identifier_type = identifier.get('identifierType', '')
        if identifier_type == 'GTIN' or identifier_type == 'UPC':
          upc = identifier.get('identifier', upc)
          break

    return upc

  def get_categories(self, product_info):
    categories = None
    d = dict()
    summary = product_info.get('summaries', [d])[0]
    if summary and 'websiteDisplayGroupName' in summary and summary['websiteDisplayGroupName']:
      categories = summary['websiteDisplayGroupName']

    return categories

  def get_specifications(self, product_info):
    specifications = []
    specs = product_info.get('specifications', product_info.get('key_specs', []))
    for spec in specs:
      if not isinstance(spec, dict):
        continue

      spec_name = spec['name'].lower()
      if spec_name.find('asin') != -1 or spec_name.find('rank') != -1 or spec_name.find('first available') != -1 or spec_name.find('amazon') != -1:
        continue

      specifications.append({'name': spec['name'], 'value': spec['value']})

    summaries = {
      'color': 'Color',
      'manufacturer': 'Manufacturer',
      'modelNumber': 'Model Number',
      'packageQuantity': 'Package Quantity',
      'partNumber': 'Part Number',
      # 'releaseDate': 'Release Date',
      'size': 'Size',
      'style': 'Style'
    }
    for summary in product_info.get('summaries', []):
      for k, v in summary.items():
        if k in summaries:
          specifications.append({'name': summaries[k], 'value': v})

    attrs = product_info.get('attributes', {})
    attr_keys = {
      'item_form': 'Form',
      'target_gender': 'Gender',
      'ingredients': 'Ingredients',
      'directions': 'Directions'
    }
    for k, v in attr_keys.items():
      if k in attrs:
        try:
          attr = attrs[k]
          if 'value' in attr and attr['value']:
            specifications.append({'name': v, 'value': attr[0]['value']})
        except Exception as e:
          logger.exception(e)
          logger.info('[AttributeError] Key: %s, Attr: %s', k, attr)

    return specifications

  def get_images(self, product_info):
    images = product_info.get('images', [])

    if isinstance(images, str):
      images = [images]

    imgs = dict()
    for image in images:
      img_url = self.opt_img(image)
      if img_url in imgs:
        imgs[img_url] += 1
      else:
        imgs[img_url] = 1

    imgs_origin = [img_url for img_url, cnt in imgs.items() if cnt == 1]
    imgs_resized = [img_url for img_url, cnt in imgs.items() if cnt > 1]

    return imgs_origin if imgs_origin else imgs_resized

  def get_weight_from_dimensions(self, dimensions):
    if len(dimensions) > 0:
      di = dimensions[0]
      return di.get('package', di.get('item', {})).get('weight', {})

  def get_weight_from_attr(self, attrs):
    return attrs.get('item_package_weight', attrs.get('item_weight', []))

  def get_weight(self, product_info):
    d = dict()
    attrs = product_info.get('attributes', d)
    dimensions = product_info.get('dimensions', [])
    weight = self.get_weight_from_dimensions(dimensions) or self.get_weight_from_attr(attrs)
    if not weight:
      return

    weight_in_pounds = None
    weight = weight[0] if isinstance(weight, list) else weight
    weight_str = '{} {}'.format(weight.get('value'), weight.get('unit'))
    try:
      weight_in_pounds = round(self.ureg(weight_str).to('lbs').m, 2) or 0.1
    except Exception as e:
      weight_in_pounds = None

    return weight_in_pounds

  def get_raw_dimensions(self, product_info):
    raw_dimensions = product_info.get('dimensions', [{}])[0]
    dimensions = raw_dimensions.get('package', raw_dimensions.get('item', {}))

    if isinstance(dimensions, list):
      dimensions = dimensions[0]
    if not isinstance(dimensions, dict):
      return None
    return dimensions

  def get_package_dimensions(self, product_info):
    d = dict()
    dimensions = product_info.get('item_package_dimensions', product_info.get('attributes', d).get('item_package_dimensions'))
    if isinstance(dimensions, list):
      dimensions = dimensions[0]
    if not isinstance(dimensions, dict):
      return None
    return dimensions


  def get_dimensions(self, product_info):
    d = dict()
    ds = {}
    dimensions = self.get_package_dimensions(product_info) or self.get_raw_dimensions(product_info)
    if not dimensions:
      return {'width': None, 'height': None, 'length': None}

    width = dimensions.get('width', d)
    if width:
      width_str = '{} {}'.format(width['value'], width['unit'])
      try:
        ds['width'] = round(self.ureg(width_str).to('inch').m, 2)
      except:
        ds['width'] = None
    else:
      ds['width'] = None

    height = dimensions.get('height', d)
    if height:
      height_str = '{} {}'.format(height['value'], height['unit'])
      try:
        ds['height'] = round(self.ureg(height_str).to('inch').m, 2)
      except:
        ds['height'] = None
    else:
      ds['height'] = None

    length = dimensions.get('length', d)
    if length:
      length_str = '{} {}'.format(length['value'], length['unit'])
      try:
        ds['length'] = round(self.ureg(length_str).to('inch').m, 2)
      except:
        ds['length'] = None
    else:
      ds['length'] = None

    return ds

  def get_sales_rank(self, product_info):
    return product_info.get('sales_rank', None)

  def is_fragile(self, product_info):
    attrs = product_info.get('attributes', {})
    if 'is_fragile' not in attrs or not attrs['is_fragile']:
      return False

    is_fragile = False
    for entry in attrs['is_fragile']:
      is_fragile = entry.get('value', False)
      if is_fragile:
        break

    return is_fragile

  def is_batteries_included(self, product_info):
    attrs = product_info.get('attributes', {})
    if 'batteries_included' not in attrs or not attrs['batteries_included']:
      return False

    batteries_required = False
    for entry in attrs.get('batteries_required', []):
      batteries_required = entry.get('value', False)
      if batteries_required:
        break

    if not batteries_required:
      return False

    batteries_included = False
    for entry in attrs['batteries_included']:
      batteries_included = entry.get('value', False)
      if batteries_included:
        break

    return batteries_included

  def is_grow_your_own_kits(self, product_info):
    d = dict()
    summary = product_info.get('summaries', [d])[0]
    if summary and 'browseClassification' in summary and summary['browseClassification']:
      if isinstance(summary['browseClassification'], dict):
        if (summary['browseClassification'].get("displayName") and
            summary['browseClassification'].get("displayName").lower().find("grow your own kits") > -1
        ):
          return True

  def is_aerosol(self, product_info):
    for item in product_info.get('hazmat', []):
      if item.get("value", "").lower().find("aerosol") > -1:
        return True

  def is_sensitive(self, product_info):
    title = product_info.get('title', '').lower()
    keywords = ["clotted cream", "smoke bomb", "woodworm destroyer"]
    for keyword in keywords:
      if title.find(keyword) > -1:
        return True
    if self.is_aerosol(product_info):
      return True

  def is_contains_food_or_beverage(self, product_info):
    attrs = product_info.get('attributes', {})
    if 'contains_food_or_beverage' not in attrs or not attrs['contains_food_or_beverage']:
      return False

    contains_food_or_beverage = False
    for entry in attrs['contains_food_or_beverage']:
      contains_food_or_beverage = entry.get('value', False)
      if contains_food_or_beverage:
        break

    return contains_food_or_beverage

  def is_liquid(self, product_info):
    attrs = product_info.get('attributes', {})
    if 'item_form' not in attrs or not attrs['item_form']:
      return False

    contains_liquid = False
    for entry in attrs['item_form']:
      item_form = entry.get('value', '').lower()
      if item_form == 'liquid':
        contains_liquid = True
        break

    return contains_liquid

  def is_alcohol(self, product_info):
    attrs = product_info.get('attributes', {})
    if 'alcohol_content' not in attrs or not attrs['alcohol_content']:
      return False

    contains_alcohol = False
    for entry in attrs['alcohol_content']:
      try:
        alcohol_percentage = float(entry.get('value', 0))
        if alcohol_percentage >= 3:
          contains_alcohol = True
          break
      except:
        pass

    return contains_alcohol

  def is_plant_or_seed(self, product_info):
    product_types = product_info.get('productTypes', product_info.get('attributes', {}).get('productTypes', None))
    if not product_types:
      title = product_info.get('title', '').lower()
      if title.find('plant') != -1:
        return True

      return False

    plant_or_seed = False
    for product_type in product_types:
      try:
        product_type_s = product_type.get('productType', product_type.get('value', '')).lower()
        if product_type_s.find('plant') != -1 or product_type_s.find('seed') != -1:
          plant_or_seed = True
          break
      except:
        pass

    return plant_or_seed

  def is_test_strip(self, product_info):
    title = product_info.get('title', '').lower()
    if title.find('testing kit') != -1 or title.find('test kit') != -1:
      return True

    product_types = product_info.get('productTypes', product_info.get('attributes', {}).get('productTypes', None))
    if not product_types:
      return False

    test_strip = False
    for product_type in product_types:
      product_type_val = product_type.get('productType', product_type.get('value', ''))
      if product_type_val.upper().find('TEST_STRIP') != -1:
        test_strip = True
        break

    return test_strip

  def is_spray(self, product_info):
    title = product_info.get('title', '').lower()
    if title.find('spray') != -1:
      return True

    item_forms = product_info.get('attributes', {}).get('item_form', None)
    if not item_forms:
      return False

    spray = False
    for item_form in item_forms:
      item_form_val = item_form.get('value', '')
      if item_form_val and item_form_val.find('Spray') != -1:
        spray = True
        break

    return spray

  def opt_img(self, img):
    if '._' in img:
      filename = os.path.basename(img)
      parts = filename.split('.')
      filename = parts[0] + '.' + parts[len(parts) - 1]
      img = os.path.dirname(img) + '/' + filename

    return img
